// Copyright (C) 2022 The Qt Company Ltd.
// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR LGPL-3.0-only OR GPL-2.0-only OR GPL-3.0-only

#ifndef Q_CTFPLUGIN_P_H
#define Q_CTFPLUGIN_P_H

//
//  W A R N I N G
//  -------------
//
// This file is not part of the Qt API.  It exists purely as an
// implementation detail.  This header file may change from version to
// version without notice, or even be removed.
//
// We mean it.
//
//

#include <private/qctf_p.h>
#include <qplugin.h>

QT_BEGIN_NAMESPACE

Q_DECLARE_INTERFACE(QCtfLib, "org.qt-project.Qt.QCtfLib")

QT_END_NAMESPACE

#endif
