// Copyright (C) 2018 The Qt Company Ltd.
// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR BSD-3-Clause

#include <QByteArrayMatcher>

//! [0]
static const auto matcher = qMakeStaticByteArrayMatcher("needle");
//! [0]
